using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Drawing;
using System.Drawing.Imaging;

/*
 * Header = 62 bytes 
1-4 = Always the same 
5 = Spacer 
6-9 = Return Key, ties back to index file 
10 = Spacer 
11-14 = Time in milliseconds since recording started 
15 = Spacer 
16-19 = Long in Mercator Meters 
20 = Spacer 
21-24 = Lat in Mercator Meters 
25 = Spacer 
26-29 = GPS Dependent, not sure what it is yet 
30 = Spacer 
31-34 = GPS Dependent, not sure what it is yet 
35 = ? 
36 = Beam # 
37-39 = ? 
40-43 = Frequency 
44-57 = ? 
58-61 = Sentence Length 
62 = Spacer, or something thats always x21 
*/

namespace Sonofagun
{

    public class Frame
    {
        internal Frame(byte[] data, Color bias, PixelFormat format)
        {
            int length = data.Length;
            Data = data;

            //Image = new Bitmap(length, 1, format);
            //int j = 0;

            //Rectangle rect = new Rectangle(new Point(0,0), Image.PhysicalDimension.ToSize());

            //BitmapData bitmapData = Image.LockBits(rect, System.Drawing.Imaging.ImageLockMode.ReadWrite, format);
            //IntPtr ptr = bitmapData.Scan0;
            //Marshal.Copy(data, 0, ptr, length);

            //Image.UnlockBits(bitmapData);

            //for (int i = 0; i < data.Length; i++)
            //{
            //    Color x = Color.FromArgb(data[i], data[i], data[i]);

            //    Image.SetPixel(j, 0, x);
            //    j++;
            //}
        }

        public Bitmap Image;
        public byte[] Data;

    }

    public class SonReader : IDisposable
    {
        private Stream source;
        private BinaryReader reader;
        private List<KeyValuePair<int, int>> index = new List<KeyValuePair<int, int>>();

        public SonReader(string filename)
        {
            if (!Path.GetExtension(filename).Equals(".son", StringComparison.InvariantCultureIgnoreCase) || !File.Exists(filename))
                throw new ArgumentException("The file name must be a valid .son file", "filename");

            source = new FileStream(filename, FileMode.Open);
            reader = new BinaryReader(source);

            string indexFile = Path.Combine(Path.GetDirectoryName(filename), Path.GetFileNameWithoutExtension(filename) + ".idx");
            if (File.Exists(indexFile))
            {
                using (FileStream indexStream = new FileStream(indexFile, FileMode.Open))
                {
                    BinaryReader indexReader = new BinaryReader(indexStream);

                    while (indexReader.BaseStream.Position < indexReader.BaseStream.Length)
                    {
                        int idx = this.BytestoInt(indexReader.ReadBytes(4));
                        int ptr = this.BytestoInt(indexReader.ReadBytes(4));

                        index.Add(new KeyValuePair<int, int>(idx, ptr));
                    }
                }
            }


        }

        public List<KeyValuePair<int, int>> Index
        {
            get { return index; }
        }

        public SonReader(Stream source)
        {
            this.source = source;
            reader = new BinaryReader(this.source);
        }

        public long Position
        {
            get { return source.Position; }
        }

        public long Length
        {
            get { return source.Length; }
        }

        public PixelFormat Format;

        public Frame ReadLine(int pos)
        {
            reader.BaseStream.Position = pos;
            byte[] block1 = reader.ReadBytes(5);
            byte[] block2 = reader.ReadBytes(5);
            byte[] time = reader.ReadBytes(4);
            reader.ReadByte();//space
            byte[] longMerc = reader.ReadBytes(4);
            reader.ReadByte();//space
            byte[] latMerc = reader.ReadBytes(4);
            reader.ReadByte();//space
            byte[] block3 = reader.ReadBytes(5);
            byte[] block4 = reader.ReadBytes(5);
            byte beam = reader.ReadByte();
            byte[] block5 = reader.ReadBytes(3);
            byte[] freq = reader.ReadBytes(4);
            byte[] block6 = reader.ReadBytes(14);
            byte[] length = reader.ReadBytes(4);
            byte twentyone = reader.ReadByte();

            int len = BytestoInt(length);
            byte[] data = reader.ReadBytes(len);

            //double dLong = LongMMtoDeg(BytestoInt(longMerc));
            //double dLat = LatMMtoDeg(BytestoInt(latMerc));

            //Debug.WriteLine(string.Format("{0}, {1}", dLat, dLong));

            return new Frame(data, Color.Blue, Format);

        }

        public Frame ReadLine()
        {
            byte[] block1 = reader.ReadBytes(5);
            byte[] block2 = reader.ReadBytes(5);
            byte[] time = reader.ReadBytes(4);
            reader.ReadByte();//space
            byte[] longMerc = reader.ReadBytes(4);
            reader.ReadByte();//space
            byte[] latMerc = reader.ReadBytes(4);
            reader.ReadByte();//space
            byte[] block3 = reader.ReadBytes(5);
            byte[] block4 = reader.ReadBytes(5);
            byte beam = reader.ReadByte();
            byte[] block5 = reader.ReadBytes(3);
            byte[] freq = reader.ReadBytes(4);
            byte[] block6 = reader.ReadBytes(14);
            byte[] length = reader.ReadBytes(4);
            byte twentyone = reader.ReadByte();

            int len = BytestoInt(length);
            byte[] data = reader.ReadBytes(len);

            //double dLong = LongMMtoDeg(BytestoInt(longMerc));
            //double dLat = LatMMtoDeg(BytestoInt(latMerc));

            //Debug.WriteLine(string.Format("{0}, {1}", dLat, dLong));

            return new Frame(data, Color.Blue, Format);

        }

        const double RADtoDEG = 57.295779513082322;
        const double EarthRad = 6356752.3142;

        private double LatMMtoDeg(int m)
        {
            double temp;
            temp = m / EarthRad;
            temp = Math.Exp(temp);
            temp = (2 * Math.Atan(temp)) - (Math.PI / 2);
            return temp * RADtoDEG;
        }


        private double LongMMtoDeg(int m)
        {

            return m / EarthRad * RADtoDEG;
        }

        struct LSTRUC
        {
            public double Lat;
            public double Lon;
        }

        private LSTRUC Merc_2_Degree(long lat, long lon)
        {
            LSTRUC T = new LSTRUC();
            double temp;
            temp = (double)lat / (double)6356752.3142;

            temp = Math.Exp(temp);
            temp = (2 * Math.Atan(temp)) - (Math.PI / 2);
            T.Lat = temp * RADtoDEG;
            T.Lon = (double)lon / (double)6356752.3142;
            T.Lon = T.Lon * RADtoDEG;
            return T;
        }

        private int BytestoInt(byte[] data)
        {
            Array.Reverse(data);
            BinaryReader r = new BinaryReader(new MemoryStream(data));
            return (int)r.ReadInt32();
        }

        #region IDisposable Members

        public void Dispose()
        {
            if (source != null)
                source.Dispose();
        }

        #endregion
    }
}
